package;

import flash.events.MouseEvent;
import flash.Lib;
import flash.text.TextField;
import flash.text.TextFieldAutoSize;
import flash.text.TextFieldType;
import openfl.Assets;
import flash.geom.Rectangle;
import flash.net.SharedObject;
import org.flixel.FlxButton;
import org.flixel.FlxCamera;
import org.flixel.FlxG;
import org.flixel.FlxPath;
import org.flixel.FlxSave;
import org.flixel.FlxSprite;
import org.flixel.FlxState;
import org.flixel.FlxText;
import org.flixel.FlxTilemap;
import org.flixel.system.input.FlxMouse;
import org.flixel.util.FlxMath;
import org.flixel.util.FlxPoint;
import org.flixel.util.FlxRect;

class PlayState extends FlxState
{
	var camera:FlxCamera;
	var background:FlxSprite;
	var map:FlxTilemap;
	var player:FlxSprite;
	var enemy:FlxSprite;
	
	inline static var TILESIZE:Int = 40;
	var speed:Int = 80;
	var gravity_accel:Float = 500;
	var jump_accel:Float = 260;
	
	//ui
	var widthField:TextField;
	var heightField:TextField;
	var speedField:TextField;
	var dragField:TextField;
	var gravityField:TextField;
	var jumpField:TextField;
	var zoomField:TextField;
	var bSpriteMode:Bool = false;
	
	override public function create():Void
	{
		// Set a background color
		FlxG.bgColor = 0xff131c1b;
		FlxG.mouse.useSystemCursor = true;
		// Show the mouse (in case it hasn't been disabled)
		//#if !FLX_NO_MOUSE
		//FlxG.mouse.show();
		//#end
		
		super.create();
		background = new FlxSprite(0, 0, Assets.getBitmapData("assets/clouds.jpg"));
		background.scale.x = background.scale.y = 3;
		background.scrollFactor = new FlxPoint(0.25, 1);
		add(background);
		map = new FlxTilemap();
		map.loadMap(Assets.getText("assets/mapCSV_Group1_Map1.txt"), Assets.getBitmapData("assets/tiles.PNG"), TILESIZE, TILESIZE,0,0,1,1);
		add(map);
		//player stuff
		player = new FlxSprite(TILESIZE * 5, (map.heightInTiles - 4) * TILESIZE);// , Assets.getBitmapData("assets/player.png"));
		player.makeGraphic(40, 80, 0xffff0000);
		player.drag.x = 640;
		player.acceleration.y = gravity_accel;
		player.maxVelocity.x = speed;
		player.maxVelocity.y = jump_accel;
		add(player);
		//enemy stuff
		enemy = new FlxSprite(TILESIZE * 25, (map.heightInTiles - 2) * TILESIZE, Assets.getBitmapData("assets/enemy.PNG"));
		add(enemy);
		//set up the camera:
		camera = new FlxCamera(0, 0, FlxG.width, FlxG.height, 1);
		camera.follow(player, FlxCamera.STYLE_PLATFORMER); //follow the player with a slight dead zone
		//camera.setBounds(0, 0, map.width, map.height); //make sure you don't go past the edge of the tilemap
		FlxG.addCamera(camera); //add the camera
		FlxG.worldBounds = new FlxRect(0, 0, map.width, map.height); //need this to tell flixel where to calculate collisions
		
		//text
		var spacing:Int = 10;
		var xOffset:Int = 500;
		var widthLabel:TextField = CreateTextField(xOffset,spacing,"Width",false);
		widthField = CreateTextField(xOffset + widthLabel.width, spacing, "40", true);
		var heightLabel:TextField = CreateTextField(xOffset, widthLabel.y + widthLabel.height + spacing, "Height", false);
		heightField = CreateTextField(xOffset + heightLabel.width, widthLabel.y + widthLabel.height + spacing, "80", true);
		var speedLabel:TextField = CreateTextField(xOffset, heightField.y+heightField.height+spacing, "speed", false);
		speedField = CreateTextField(xOffset + speedLabel.width, heightField.y + heightField.height + spacing, "80", true);
		var dragLabel:TextField = CreateTextField(xOffset, speedField.y + speedField.height + spacing, "drag");
		dragField = CreateTextField(xOffset + dragLabel.width, speedField.y + speedField.height + spacing, "640", true);
		var gravityLabel:TextField = CreateTextField(xOffset, dragLabel.y + dragLabel.height + spacing, "gravity accel.");
		gravityField = CreateTextField(xOffset + gravityLabel.width, dragLabel.y + dragLabel.height + spacing, "500", true);
		var jumpLabel = CreateTextField(xOffset, gravityLabel.y + gravityLabel.height + spacing, "jump accel.");
		jumpField = CreateTextField(xOffset + gravityLabel.width, gravityLabel.y + gravityLabel.height + spacing, "260", true);
		var zoomLabel = CreateTextField(xOffset, jumpField.y + jumpField.height + spacing, "zoom");
		zoomField = CreateTextField(xOffset + zoomLabel.width, jumpField.y + jumpField.height + spacing, "1.0", true);
		var update:TextField = CreateTextField(xOffset,zoomField.y+zoomField.height+spacing,"UPDATE",false,true);
		update.addEventListener(MouseEvent.CLICK, OnUpdate, false, 0, true);
		var togglePlayer:TextField = CreateTextField(xOffset, update.y + update.height + spacing, "Toggle Player", false, true);
		togglePlayer.addEventListener(MouseEvent.CLICK, OnTogglePlayer, false, 0, true);
	}
	
	private function CreateTextField(xPos:Float,yPos:Float,text:String,inputField:Bool=false, button:Bool=false):TextField
	{
		var t:TextField = new TextField();
		t.x = xPos;
		t.y = yPos;
		t.autoSize = TextFieldAutoSize.LEFT;
		t.border = true;
		t.borderColor = 0xffff00;
		t.text = text;
		if (!inputField)
		{
			if (button) t.mouseEnabled = true;
			else t.mouseEnabled = false;
			t.background = true;
			t.backgroundColor = 0xff0000;
			t.selectable = false;
			t.type = TextFieldType.DYNAMIC;
		}
		else
		{
			t.selectable = true;
			t.background = false;
			t.type = TextFieldType.INPUT;
		}
		FlxG.stage.addChild(t);
		return t;
	}
	
	private function OnUpdate(e:MouseEvent):Void
	{
		UpdatePlayer();
		speed = Std.parseInt(speedField.text);
		gravity_accel = Std.parseInt(gravityField.text);
		jump_accel = Std.parseInt(jumpField.text);
		player.drag.x = Std.parseInt(dragField.text);
		player.acceleration.y = gravity_accel;
		player.maxVelocity.x = speed;
		player.maxVelocity.y = jump_accel;
		camera.zoom = Std.parseFloat(zoomField.text);
		camera.follow(player, FlxCamera.STYLE_PLATFORMER); //follow the player with a slight dead zone
	}
	
	private function OnTogglePlayer(e:MouseEvent):Void
	{
		bSpriteMode = !bSpriteMode;
		UpdatePlayer();
	}
	
	private function UpdatePlayer():Void
	{
		if (bSpriteMode)
		{
			player.loadGraphic(Assets.getBitmapData("assets/player.png"), false, false, 80, 80);
			player.width = Std.parseInt(widthField.text);
			player.height = Std.parseInt(heightField.text);
		}
		else player.makeGraphic(Std.parseInt(widthField.text), Std.parseInt(heightField.text), 0xffff0000);
	}
	
	/**
	 * Function that is called when this state is destroyed - you might want to 
	 * consider setting all objects this state uses to null to help garbage collection.
	 */
	override public function destroy():Void
	{
		super.destroy();
	}

	/**
	 * Function that is called once every frame.
	 */
	override public function update():Void
	{
		super.update();
		FlxG.overlap(player, enemy, OnHit);
		FlxG.collide(player, map); //keep the player on the tilemap
		//player stuff
		//calculate current speed and acceleration
		player.acceleration.x = 0;
		if(FlxG.keys.LEFT) {
			player.acceleration.x = -player.drag.x;
		}
		else if(FlxG.keys.RIGHT) {
			player.acceleration.x = player.drag.x;
		}
		if(FlxG.keys.justPressed("SPACE") && player.velocity.y==0) {
			player.velocity.y = -jump_accel;
		}
	}
	
	private function OnHit(player:FlxSprite,enemy:FlxSprite):Void
	{
		//trace("hit");
		player.y = player.y - 20;
		player.x = player.x - 20;
		//player.velocity.y = -200;
	}
}